/*
 * Copyright (c) 2007 Kathryn Huxtable
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: RSASignature.java 1074 2007-04-08 19:27:55Z huxtable $
 */
package org.kathrynhuxtable.middleware.shibshim.util;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

/**
 * Simple wrapper around AES encrypt/decrypt operations.
 */
public class AESCrypt {
    
    /**
     * Private constructor defeats instantiation.
     */
    private AESCrypt() {
    }

    /**
     * Make a new AES key.
     * 
     * @return the new Base64 encoded AES key.
     * @throws AESCryptException if an error occurs.
     */
    public static String makeAESKey() throws AESCryptException {
        String aesKey = null;

        try {
            KeyGenerator keygen = KeyGenerator.getInstance("AES");
            SecretKey secretKey = keygen.generateKey();
            aesKey = Base64.encodeBytes(secretKey.getEncoded());
        } catch (NoSuchAlgorithmException e) {
            throw new AESCryptException("No such algorithm (AES): " + e);
        }

        return aesKey;
    }

    /**
     * Encrypt clear text using an AES key.
     * 
     * @param clearText
     *            the clear text.
     * @param aesKey
     *            the Base64 encoded AES key.
     * @return the cipher text.
     * @throws AESCryptException
     *             if an error occurs.
     */
    public static byte[] encrypt(byte[] clearText, String aesKey) throws AESCryptException {
        // Decode Base64 data into a private key.
        SecretKeySpec secretKeySpec = new SecretKeySpec(Base64.decode(aesKey), "AES");

        // Initialize the cipher instance for encryption with the private key.
        Cipher desCipher = null;
        String transform = "AES/ECB/PKCS5Padding";
        try {
            desCipher = Cipher.getInstance(transform);
        } catch (NoSuchAlgorithmException e) {
            throw new AESCryptException("No such algorithm (" + transform + "): " + e);
        } catch (NoSuchPaddingException e) {
            throw new AESCryptException("No such padding (" + transform + "): " + e);
        }

        try {
            // Initialize the cipher for encryption
            desCipher.init(Cipher.ENCRYPT_MODE, secretKeySpec);
        } catch (InvalidKeyException e) {
            throw new AESCryptException("Invalid key", e);
        }

        // Encrypt the cleartext
        byte[] cipherText = null;
        try {
            cipherText = desCipher.doFinal(clearText);
        } catch (IllegalStateException e) {
            throw new AESCryptException("Illegal state", e);
        } catch (IllegalBlockSizeException e) {
            throw new AESCryptException("Illegal block size", e);
        } catch (BadPaddingException e) {
            throw new AESCryptException("Bad padding", e);
        }

        return cipherText;
    }

    /**
     * Decrypt AES-encoded cipher text.
     * 
     * @param cipherText
     *            the cipher text.
     * @param aesKey
     *            the Base64 encoded AES key.
     * @return the clear text.
     * @throws AESCryptException
     *             if an error occurs.
     */
    public static byte[] decrypt(byte[] cipherText, String aesKey) throws AESCryptException {
        // Decode Base64 data into a private key.
        SecretKeySpec secretKeySpec = new SecretKeySpec(Base64.decode(aesKey), "AES");

        // Initialize the cipher instance for encryption with the private key.
        Cipher aesCipher = null;
        String transform = "AES/ECB/PKCS5Padding";
        try {
            aesCipher = Cipher.getInstance(transform);
        } catch (NoSuchAlgorithmException e) {
            throw new AESCryptException("No such algorithm (" + transform + "): " + e);
        } catch (NoSuchPaddingException e) {
            throw new AESCryptException("No such padding (" + transform + "): " + e);
        }

        try {
            // Initialize the cipher for decryption
            aesCipher.init(Cipher.DECRYPT_MODE, secretKeySpec);
        } catch (InvalidKeyException e) {
            throw new AESCryptException("Invalid key", e);
        }

        // Decrypt the ciphertext
        byte[] clearText = null;
        try {
            clearText = aesCipher.doFinal(cipherText);
        } catch (IllegalStateException e) {
            throw new AESCryptException("Illegal state", e);
        } catch (IllegalBlockSizeException e) {
            throw new AESCryptException("Illegal block size", e);
        } catch (BadPaddingException e) {
            throw new AESCryptException("Bad padding", e);
        }

        return clearText;
    }

    /**
     * Main program to generate a new AES key.
     * 
     * @param args
     *            the typical Java command-line args array.
     */
    public static void main(String[] args) {
        try {
            System.out.println(makeAESKey());
        } catch (AESCryptException e) {
            e.printStackTrace();
            System.exit(1);
        }
        System.exit(0);
    }
}
