/*
 * Copyright (c) 2007 Kathryn Huxtable
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: RSASignature.java 1166 2007-04-15 17:00:00Z kathryn5 $
 */
package org.kathrynhuxtable.middleware.shibshim.util;

import java.io.ByteArrayInputStream;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;

/**
 * Simple wrapper around RSA sign and verify operations.
 */
public class RSASignature {
    
    /**
     * Private constructor defeats instantiation.
     */
    private RSASignature() {
    }

    /**
     * Digitally sign document bytes with a private key.
     * 
     * @param documentBytes
     *            the document bytes to be signed
     * @param pemKey
     *            the PEM encoded private key in PKCS#8 format
     * @return base64 encoded signature block
     * @throws RSASignatureException
     *             if a problem occurs with the verification operation.
     */
    public static byte[] sign(byte[] documentBytes, String pemKey) throws RSASignatureException {
        // Strip trailing blanks and the BEGIN and END markers.
        pemKey = pemKey.replaceAll(" *\r?\n+", "\n");
        pemKey = pemKey.replaceFirst("^-----BEGIN PRIVATE KEY-----\n", "");
        pemKey = pemKey.replaceFirst("\n-----END PRIVATE KEY-----\n", "\n");

        // Decode PEM data into a private key.
        PrivateKey privateKey = null;
        try {
            byte[] pemBytes = Base64.decode(pemKey);
            KeyFactory kf = KeyFactory.getInstance("RSA");
            PKCS8EncodedKeySpec kspec = new PKCS8EncodedKeySpec(pemBytes);
            privateKey = kf.generatePrivate(kspec);
        } catch (NoSuchAlgorithmException e) {
            throw new RSASignatureException("No such algorithm (RSA)", e);
        } catch (InvalidKeySpecException e) {
            throw new RSASignatureException("Invalid key specification", e);
        }

        // Initialize the signature instance for signing with the private key.
        Signature signature = null;
        try {
            signature = Signature.getInstance("SHA1withRSA");
            signature.initSign(privateKey);
        } catch (NoSuchAlgorithmException e) {
            throw new RSASignatureException("No such algorithm (SHA1withRSA)", e);
        } catch (InvalidKeyException e) {
            throw new RSASignatureException("Invalid key while signing text", e);
        }

        // Add the text to be signed.
        try {
            signature.update(documentBytes);
        } catch (SignatureException e) {
            throw new RSASignatureException("Signature exception adding text", e);
        }

        // Sign encode the result.
        byte[] sigBytes = null;
        try {
            sigBytes = signature.sign();
        } catch (SignatureException e) {
            throw new RSASignatureException("Signature exception signing text", e);
        }

        return sigBytes;
    }

    /**
     * Verify that the document bytes were signed by the key for a public
     * certificate.
     * 
     * @param documentBytes
     *            the document bytes to verify
     * @param sigBlock
     *            the signature block to verify
     * @param pemCert
     *            the PEM encoded certificate to verify
     * @return true if the signature was signed correctly
     * @throws RSASignatureException
     *             if a problem occurs with the signing operation.
     */
    public static boolean verify(byte[] documentBytes, byte[] sigBlock, String pemCert) throws RSASignatureException {
        // Strip any trailing blanks.
        pemCert = pemCert.replaceAll(" *\r?\n+", "\n");

        // Decode the PEM format cert into an X509Certificate object.
        X509Certificate cert = null;
        try {
            CertificateFactory cf = CertificateFactory.getInstance("X.509");
            cert = (X509Certificate) cf.generateCertificate(new ByteArrayInputStream(pemCert.getBytes()));
        } catch (CertificateException e) {
            throw new RSASignatureException("Certificate exception", e);
        }

        // Initialize the signature instance for verifying with the public cert.
        Signature signature = null;
        try {
            signature = Signature.getInstance("SHA1withRSA");
            signature.initVerify(cert);
        } catch (NoSuchAlgorithmException e) {
            throw new RSASignatureException("No such algorithm (SHA1withRSA) exception", e);
        } catch (InvalidKeyException e) {
            throw new RSASignatureException("Invalid key while verifying", e);
        }

        // Verify the signature.
        boolean status = false;
        try {
            signature.update(documentBytes);
            status = signature.verify(sigBlock);
        } catch (SignatureException e) {
            throw new RSASignatureException("Signature exception while verifying", e);
        }

        return status;
    }
}
